@@ -3,15 +3,43 @@ import { RobotModel } from './robot_model';
 import $ from 'jquery';
 import _isNumber from 'lodash/isNumber';
 import _random from 'lodash/random';
+import _padStart from 'lodash/padStart';
+
+const walls_config: { [key: string]: any } = {
+  normal: {
+    x: 50,
+    y: 25,
+    stroke: 'darkred',
+    strokeWidth: 10,
+  },
+
+  removable: {
+    x: 50,
+    y: 25,
+    stroke: '#de1738',
+    strokeWidth: 10,
+  },
+
+  goal: {
+    x: 50,
+    y: 25,
+    stroke: 'darkred',
+    strokeWidth: 7,
+    dash: [10, 10],
+  },
+};
 export class WorldModel {
   rows: number;
   cols: number;
-  walls: any;
+  vwalls: any;
+  hwalls: any;
   robots: any;
   width: number;
   height: number;
   bs: number;
   objects: any;
+  tileMap: any;
+  tiles: any;
   ui: {
     wrapper: HTMLDivElement;
     stage?: Konva.Stage;
@@ -19,6 +47,7 @@ export class WorldModel {
       main: Konva.Layer;
       line: Konva.Layer;
       msg: Konva.Layer;
+      bg: Konva.Layer;
     };
   };
 
@@ -26,14 +55,26 @@ export class WorldModel {
     this.init_ui();
   }
 
-  init(rows: number, cols: number, walls = {}, robots = [], objects = {}) {
+  init(
+    rows: number,
+    cols: number,
+    vwalls = [],
+    hwalls = [],
+    robots = [],
+    objects = {},
+    tileMap = {},
+    tiles = []
+  ) {
     this.rows = rows;
     this.cols = cols;
-    this.walls = walls;
+    this.vwalls = vwalls;
+    this.hwalls = hwalls;
     this.bs = 50; //box_size
     this.height = rows * this.bs;
     this.width = cols * this.bs;
     this.objects = objects;
+    this.tileMap = tileMap;
+    this.tiles = tiles;
     this.draw_canvas();
 
     this.robots = robots.map(
@@ -78,6 +119,7 @@ export class WorldModel {
     this.ui = {
       wrapper,
       layers: {
+        bg: new Konva.Layer(),
         main: new Konva.Layer(),
         line: new Konva.Layer(),
         msg: new Konva.Layer(),
@@ -93,6 +135,7 @@ export class WorldModel {
       height: this.height + this.bs + padding,
     });
 
+    stage.add(this.ui.layers.bg);
     stage.add(this.ui.layers.main);
     stage.add(this.ui.layers.line);
     stage.add(this.ui.layers.msg);
@@ -179,7 +222,30 @@ export class WorldModel {
     this.ui.layers.main.draw();
   }
 
-  draw_custom(obj_name: string, x: number, y: number, val: number) {}
+  draw_custom(obj_name: string, x: number, y: number, val: number) {
+    let imagePath = this.tileMap[obj_name];
+    let [cx, cy] = this.point2cxy(x + 1, y);
+    Konva.Image.fromURL(imagePath, (node: Konva.Image) => {
+      node.setAttrs({
+        x: cx + 5,
+        y: cy - 25,
+        width: 40,
+        height: 40,
+        name: `obj-${x}-${y}-img`,
+      });
+      this.ui.layers.main.add(node);
+      this.ui.layers.main.batchDraw();
+    });
+
+    let num = new Konva.Text({
+      text: `${val}`,
+      x: cx + 32,
+      y: cy + 2,
+      fontSize: 16,
+      name: `obj-${x}-${y}-text`,
+    });
+    this.ui.layers.main.add(num);
+  }
 
   parse_value(val: number | string) {
     if (!val) return 0;
@@ -208,6 +274,33 @@ export class WorldModel {
     this.draw_cols();
     this.draw_rows();
     this.draw_walls();
+    this.draw_tiles();
+  }
+
+  _draw_tile(x: number, y: number, tile: string) {
+    let [cx, cy] = this.point2cxy(x, y);
+    let imagePath = this.tileMap[tile];
+    Konva.Image.fromURL(imagePath, (node: Konva.Image) => {
+      node.setAttrs({
+        x: cx + 50,
+        y: cy - 25,
+        width: 50,
+        height: 50,
+        name: `obj-${x}-${y}-tilebg`,
+      });
+      this.ui.layers.bg.add(node);
+      this.ui.layers.bg.batchDraw();
+    });
+  }
+
+  draw_tiles() {
+    this.tiles.forEach((list: any, row: number) => {
+      list.forEach((tile: any, col: number) => {
+        if (!!tile) {
+          this._draw_tile(row + 1, col + 1, tile);
+        }
+      });
+    });
   }
 
   draw_cols() {
@@ -268,25 +361,22 @@ export class WorldModel {
     return [(x - 1) * this.bs, this.height - (y - 1) * this.bs];
   }
 
-  add_wall(x: number, y: number, dir: string) {
+  draw_wall(x: number, y: number, dir: string, wall_type: string = 'normal') {
+    let config = walls_config[wall_type];
     let border = null;
     let [cx, cy] = this.point2cxy(x, y);
     if (dir === 'east') {
       border = new Konva.Line({
-        x: 50,
-        y: 25,
-        stroke: 'darkred',
-        strokeWidth: 10,
+        ...config,
+        name: `vwall-${x}-${y}`,
         points: [cx + this.bs, cy - this.bs, cx + this.bs, cy],
       });
     }
 
     if (dir === 'north') {
       border = new Konva.Line({
-        x: 50,
-        y: 25,
-        stroke: 'darkred',
-        strokeWidth: 10,
+        name: `hwall-${x}-${y}`,
+        ...config,
         points: [cx, cy - this.bs, cx + this.bs, cy - this.bs],
       });
     }
@@ -294,16 +384,54 @@ export class WorldModel {
     if (border) this.ui.layers.main.add(border);
   }
 
-  draw_walls() {
-    for (const coord in this.walls) {
-      if (Object.prototype.hasOwnProperty.call(this.walls, coord)) {
-        const walls = this.walls[coord];
+  remove_wall(x: number, y: number, dir: string) {
+    if (dir !== 'north' && dir !== 'east') return;
+    let wall = this.ui.layers.main.find(
+      `.${dir === 'north' ? 'hwall' : 'vwall'}-${x}-${y}`
+    )[0];
+    if (wall) {
+      wall.destroy();
+    }
+    this.ui.layers.main.draw();
+  }
 
-        let [x, y] = coord.split(',').map((co) => parseInt(co));
-        walls.map((dir: string) => {
-          this.add_wall(x, y, dir);
-        });
+  draw_typed_wall(x: number, y: number, dir: string, val: number) {
+    let [isGoal, isRemovable, isWall] = _padStart(
+      Number(val).toString(2),
+      3,
+      '0'
+    );
+
+    if (parseInt(isWall)) {
+      if (parseInt(isRemovable)) {
+        this.draw_wall(x, y, dir, 'removable');
+      } else {
+        this.draw_wall(x, y, dir, 'normal');
       }
+    } else if (parseInt(isGoal)) {
+      this.draw_wall(x, y, dir, 'goal');
     }
   }
+
+  draw_walls() {
+    this.hwalls.forEach((hw: any, i: number) => {
+      hw.forEach((val: number, j: number) => {
+        if (val) {
+          this.draw_typed_wall(i, j, 'north', val);
+        } else {
+          this.remove_wall(i, j, 'north');
+        }
+      });
+    });
+
+    this.vwalls.forEach((vw: any, i: number) => {
+      vw.forEach((val: number, j: number) => {
+        if (val) {
+          this.draw_typed_wall(i, j, 'east', val);
+        } else {
+          this.remove_wall(i, j, 'east');
+        }
+      });
+    });
+  }
 }
