@@ -11,11 +11,14 @@ class PositionGoal():
 
 class WallGoal():
     def __init__(self, config):
-        self.walls = config.get("walls", [])
-        self.xy = config.get('xy')
+        self.directions = config.get("walls", [])
+        self.x  = config.get("x")
+        self.y = config.get("y")
     
     def is_completed(self, bot, world=None):
-        return sorted(world.added_walls.get(self.xy, [])) == sorted(self.walls) 
+        # return sorted(world.added_walls.get(self.xy, [])) == sorted(self.walls)
+        cell = world.cells[self.x - 1][self.y - 1]
+        return all(cell.has_block(direction) for direction in self.directions)
 
     def msg(self):
         return "Build walls at:  {}".format(self.xy)
@@ -23,16 +26,17 @@ class WallGoal():
 class ObjectGoal():
     def __init__(self, config):
         self.obj_name = config.get("obj_name")
-        self.xy  = config.get("xy")
+        self.x  = config.get("x")
+        self.y = config.get("y")
         self.val  = config.get('val')
 
     def is_completed(self, bot, world=None):
-        obj = bot.collections.get(self.xy, {})
+        obj = bot.collections.get("{},{}".format(self.x, self.y), {})
         val = obj.get(self.obj_name, 0)
         return self.val == val
 
     def msg(self):
-        return "Picked {} at: {}".format(self.obj_name, self.xy)
+        return "Picked {} at: {},{}".format(self.obj_name, self.x, self.y)
 
 class_list = {
     "position": PositionGoal,
