@@ -62,20 +62,25 @@ class Robot():
     
     def wall_in_front(self):
         return not self.is_front_clear()
+    
+    def cell(self):
+        return self.world_model.cells[self.x - 1][self.y - 1 ]
 
     def build_wall(self):
-        if self.world_model.is_clear(self.x, self.y, self.dir):
-            if self.dir == 0:
-                self.world_model.add_wall(self.x, self.y, "east")
-            if self.dir == 1:
-                self.world_model.add_wall(self.x, self.y, "north")
-            if self.dir == 2:
-                self.world_model.add_wall(self.x - 1, self.y, "east")
-            if self.dir == 3:
-                self.world_model.add_wall(self.x, self.y - 1, "north")
+        
+        if self.is_front_clear():
+            self.cell().add_wall(self.dir, self.world.js_call)
         else:
             print_error("Wall Already exists")
             self.world.js_call("error", ["Wall already exists"])
+    
+    def remove_wall(self):
+        if not self.is_front_clear():
+            self.cell().remove_wall(self.dir, self.world.js_call)
+        else:
+            print_error("No Wall exists")
+            self.world.js_call("error", ["No Wall exists"])
+
 
     def set_trace(self, color='red'):
         self.world.js_call('set_trace', [self.index, color])
